#include <AFMotor.h>
#include <NewPing.h>
#include <SoftwareSerial.h>

#define TRIG_PIN A2
#define ECHO_PIN A3
#define MAX_DISTANCE 200
#define MIN_DISTANCE 20
#define MAX_SPEED 190

// pins for the SoftwareSerial communication with HC-05
#define BT_RX_PIN 2
#define BT_TX_PIN 13

// SoftwareSerial object for HC-05
SoftwareSerial bluetoothSerial(BT_RX_PIN, BT_TX_PIN);

// declare motors for each wheel
AF_DCMotor motor1(1, MOTOR12_1KHZ);
AF_DCMotor motor2(2, MOTOR12_1KHZ);
AF_DCMotor motor3(3, MOTOR34_1KHZ);
AF_DCMotor motor4(4, MOTOR34_1KHZ);

// ultrasonic ranging module
NewPing sonar(TRIG_PIN, ECHO_PIN, MAX_DISTANCE);

// distance to an obstacle in front of the car
int distance;
boolean stop = true;

void setup() {
  // put your setup code here, to run once:

  // set the baud rate for SoftwareSerial communication
  bluetoothSerial.begin(9600);
  Serial.begin(9600);
  delay(200);

  distance = readPing();
  delay(100);
  distance = readPing();
  delay(100);
  distance = readPing();
  delay(100);
  distance = readPing();
  delay(100);
}

void loop() {
  // put your main code here, to run repeatedly:
  moveForward();
  delay(200);

  // car needs to stop if it is closer than 20 cm to something
  if (distance <= MIN_DISTANCE) {
    moveStop();
    delay(100);
    moveBackward();
    delay(400);
    moveStop();
    delay(200);

    // choose random direction for the car
    int turn = random(2);

    while (distance <= MIN_DISTANCE) {
      if (turn % 2) {
        turnRight();
        moveStop();
        delay(200);
      } else {
        turnLeft();
        moveStop();
        delay(200);
      }

      distance = readPing();
    }
  }
  
  distance = readPing();
}

int readPing() {
  delay(70);
  int dist = sonar.ping_cm();
  return dist == 0 ? 250 : dist;
}

void moveStop() {
  motor1.run(RELEASE);
  motor2.run(RELEASE);
  motor3.run(RELEASE);
  motor4.run(RELEASE);
} 
  
void moveForward() {
  if (stop) {
    stop = false;
    
    motor1.run(FORWARD);
    motor2.run(FORWARD);
    motor3.run(FORWARD);
    motor4.run(FORWARD);
   
    for (int speed = 0; speed < MAX_SPEED; speed +=2) {
      motor1.setSpeed(speed);
      motor2.setSpeed(speed);
      motor3.setSpeed(speed);
      motor4.setSpeed(speed);

      delay(5);
    }
  }
}

void moveBackward() {
  stop = true;

  motor1.run(BACKWARD);
  motor2.run(BACKWARD);
  motor3.run(BACKWARD);
  motor4.run(BACKWARD);

  for (int speed = 0; speed < MAX_SPEED; speed +=2) {
    motor1.setSpeed(speed);
    motor2.setSpeed(speed);
    motor3.setSpeed(speed);
    motor4.setSpeed(speed);

    delay(5);
  }
}  

void turnRight() {
  motor1.run(FORWARD);
  motor2.run(BACKWARD);
  motor3.run(BACKWARD);
  motor4.run(FORWARD);

  delay(500);
  
  motor1.run(FORWARD);
  motor2.run(FORWARD);
  motor3.run(FORWARD);
  motor4.run(FORWARD);
} 
 
void turnLeft() {
  motor1.run(BACKWARD);
  motor2.run(FORWARD);
  motor3.run(FORWARD);
  motor4.run(BACKWARD);

  delay(500);

  motor1.run(FORWARD);
  motor2.run(FORWARD);
  motor3.run(FORWARD);
  motor4.run(FORWARD);
}
